#!/bin/bash

trap "echo " SIGINT
export DEBIAN_FRONTEND=noninteractive
DIR="/tmp/update_to_amd64"
DIR_DOWNLOAD="$DIR/download_pkg"
LOG_ERR="$DIR/upgrade.err"
LOG="$DIR/upgrade.log"

INSTALL_LIST='
libc-bin
perl-base
libcap-ng0
libaudit1
libgpg-error0
libgcrypt20
libsystemd0
libpam0g
libpam-modules
lib
dash
bash
e2fslibs
e2fsprogs
'

__COLOR() {

	[ ! "$1" ] && return 0

	local color_settings="$3"
	local color_end="\e[0m"

	case "$TERM" in
		xterm-color|*-256color)
			case "$2" in
				line)      C="\e[4m" ;;
				inverted)  C="\e[7m" ;;
				blink)     C="\e[5m" ;;
				bold)      C="\e[1m" ;;
				red)       C="\e[31m" ;;
				red_l)     C="\e[91m" ;;
				green)     C="\e[32m" ;;
				green_l)   C="\e[92m" ;;
				yellow)    C="\e[33m" ;;
				yellow_l)  C="\e[93m" ;;
				blue)      C="\e[34m" ;;
				blue_l)    C="\e[94m" ;;
				cyan)      C="\e[36m" ;;
				cyan_l)    C="\e[96m" ;;
				gray)      C="\e[90m";;
				gray_l)    C="\e[37m";;
				magenta)   C="\e[35m" ;;
				magenta_l) C="\e[95m" ;;
				white)     C="\e[97m" ;;
				red2)      C="\e[37;1;41m" ;;
				*)         C="\e[39m" ;;
			esac
			echo -e${color_settings} "${C}${1}${color_end}"
		;;
		*)
			echo -e${color_settings} "${1}"
		;;
	esac

}


__GET_SELECT() {

	local G_SELECT=''
	local G_DEFAULT="$1"

	read G_SELECT
		case "$G_SELECT" in
			YES|yes|Yes|y|Y) return 1 ;;
			NO|no|No|n|N)    return 0 ;;
			*)
				if [ "$G_DEFAULT" ] && [ "$G_SELECT" == '' ]; then
					return "$G_DEFAULT"
				else
					__COLOR "wrong input: (Press enter to continue...)" 'red2' && read ENTER
					return 99
				fi
			;;
		esac

}


__WARNING() {

	if [ -z "$(uname -r | grep -i amd64)" ]; then
		__COLOR " WARNING!!! after crossgrade from 32-bit to 64-bit architecture your PBX settings will be drop to default (we a strong recommended doing backup) " "red2"
		echo -e "\n\n\n"
		__COLOR "Enter to continue: y/n (default n)\n" "inverted" "n"
		__GET_SELECT "0"
		[ "$?" != "1" ] && exit
		__BACKUP_SYSTEM "create"
		sudo /usr/sbin/wmsreset
	fi

}


__BACKUP_SYSTEM() {

	local latest_backup="$(ls -t /mnt/backups/*.tar.gz 2> /dev/null | head -n 1)"

	case "$1" in
		create)
			__COLOR "\n------------------------" "bold"
			__COLOR "Create backup:" "yellow_l"
			__COLOR "------------------------\n" "bold"
			sleep 2
			/usr/sbin/wms_backup_create
		;;
		apply)
			if [ "$latest_backup" ]; then
				__COLOR "\n------------------------" "bold"
				__COLOR "Apply backup ($latest_backup):" "yellow_l"
				__COLOR "------------------------\n" "bold"
				sleep 2
				/usr/sbin/wms_applybackup "$latest_backup"
			fi
	 ;;
	esac

}


__SYSTEM_PREPARING() {

	local pkg_for_remove='wms-interface-sqlite3 slapd openvpn wildix-base wildix-php7.1'

	__COLOR "\n------------------------" "bold"
	__COLOR "Preparing to switch i386 to amd64:" "yellow_l"
	__COLOR "------------------------\n" "bold"
	sleep 2

	apt-get clean
	apt-get update
	for pkg in $pkg_for_remove; do
		apt-get -yq autoremove $pkg
	done
	apt-get -yq autoremove
	apt-get dist-upgrade -yq
	apt-get clean
	## cleanup old lua profiles
	rm -f /etc/user.d/*.lua

}


__AMD64_DPKG_UPDATING() {

	__COLOR "\n------------------------" "bold"
	__COLOR "Install apt-utils (amd64):" "yellow_l"
	__COLOR "------------------------\n" "bold"
	sleep 2

	apt-get clean
	apt-get -yq --no-install-recommends --download-only install dpkg:amd64 apt:amd64 apt-utils:amd64
	dpkg -i /var/cache/apt/archives/*.deb
	dpkg -i /var/cache/apt/archives/dpkg*.deb
		if [ "$?" -ne "0" ]; then
			echo "ERROR: install apt-utils (exit)"
			exit
		fi
	apt-get clean
	apt-get update

}


__AMD64_BASE_DOWNLOADING() {

	__COLOR "\n------------------------" "bold"
	__COLOR "Downloading all packages (amd64):" "yellow_l"
	__COLOR "------------------------\n" "bold"
	sleep 2

	cd "$DIR_DOWNLOAD"

	for pkg in $(dpkg -l | awk '/^ii/{print $2"|"$3}' | grep -iv "wildix" | sed 's/:i386//g;s/:amd64//g' | sort -u); do
		local pkg_info=($(sed 's/|/ /g' <<< $pkg))
		apt-get download ${pkg_info[0]}:amd64=${pkg_info[1]} &> /dev/null
		[ "$?" -eq "0" ] && echo "Downloading: ${pkg_info[0]}:amd64=${pkg_info[1]}" || echo "ERROR: Downloading ${pkg_info[0]}:amd64=${pkg_info[1]}"
	done

}


__AMD64_BASE_INSTALLING() {

	__COLOR "\n------------------------" "bold"
	__COLOR "Install binaries (amd64):" "yellow_l"
	__COLOR "------------------------\n" "bold"
	sleep 2

	## fixes for installing pkg
	> /etc/ssh/sshd_config

	## install new amd64 packages
	for pkg in $INSTALL_LIST; do
		dpkg -i $DIR_DOWNLOAD/${pkg}*.deb
	done

	dpkg -i $DIR_DOWNLOAD/*.deb
	cd / && rm -rf "$DIR_DOWNLOAD"

	apt-get -yq install -f
	apt-get -yq autoremove

	__COLOR "\n------------------------" "bold"
	__COLOR "Deleting packages (i386):" "yellow_l"
	__COLOR "------------------------\n" "bold"
	sleep 2

	for pkg in $(dpkg -l | grep " i386 " | awk '/^ii/{print $2}' | sort -u); do
		apt-get -yq autoremove $pkg &> /dev/null
		[ "$?" -eq "0" ] && echo "i386-pkg deleted: $pkg"
	done

}


__AMD64_WMS_INSTALLING() {

	__COLOR "\n------------------------" "bold"
	__COLOR "Installing wms-interface (amd64):" "yellow_l"
	__COLOR "------------------------\n" "bold"
	sleep 2

	rm -rf  /rw2/var/lib/snmp/* /rw2/var/lib/ldap /var/lib/ldap

	## install wms-interface
	apt-get clean
	apt-get install wms-wildix2-firmwares wms-wildixair-firmwares wms-wildixata-firmwares wms-wildixfxs2-firmwares wms-wildixfxx-firmwares wms-wildixgsm-firmwares -yq
	apt-get clean
	apt-get install monit sudo -yq
	apt-get install wildix-base -yq
	apt-get clean
	apt-get install wms-interface-sqlite3 -yq
	apt-get clean
	apt-get dist-upgrade -yq
	apt-get clean
	[ -f /var/www/upgrade/upgrade_stage2.php ] && php /var/www/upgrade/upgrade_stage2.php
	sync

	## remove old architecture
	case $(dpkg --print-foreign-architectures) in
		i386) dpkg --remove-architecture i386 ;;
	esac

}


__AMD64_KERNEL_SWITH() {

	__COLOR "\n------------------------" "bold"
	__COLOR "Check kernel architecture:" "yellow_l"
	__COLOR "------------------------\n" "bold"
	sleep 2

	case $(dpkg --print-foreign-architectures) in
		amd64)
			__COLOR "AMD64 architectures: OK" "green_l"
		;;
		*)
			__COLOR "AMD64 architectures: INSTALLING..." "red"
			dpkg --add-architecture amd64
			apt-get update
		;;
	esac

	if [ -z "$(uname -r | grep -i amd64)" ]; then
		__COLOR "AMD64 kernel: INSTALLING..." "red"
		sleep 2
		apt-get install linux-image-amd64:amd64 -yq
		apt-get clean
		dpkg -l | grep linux-image | awk '/-686-/{print $2}' | xargs apt-get autoremove -yq --purge
		__COLOR "\n------------------------" "bold"
		__COLOR "Kernel was upgraded, you need reboot PBX and run script again to continued upgrade to amd64." "yellow_l"
		__COLOR "------------------------" "bold"
		__COLOR "Press any key to reboot:" "inverted" "n"
		read ans
		__COLOR "reboot..." "bold"
		sleep 5 && reboot
	else
		__COLOR "AMD64 kernel: OK" "green_l"
	fi

}


if [ -z "$(grep "^9." /etc/debian_version)" ]; then
	__COLOR "ERROR: You try run this script on wms 3.xx" "red_l"
	exit
fi

if [ "$(dpkg -l | awk '/wms-interface-sqlite3/{print $4}')" != "amd64" ]; then
	__WARNING
	echo "deb [arch=amd64] https://wmp.wildix.com/debian/get_w.php?want= rel40dev main" > /etc/apt/sources.list
	__AMD64_KERNEL_SWITH
	if [ "$(dpkg -l | awk '/dpkg/{print $4}')" != "amd64" ]; then
		rm -rf "$DIR_DOWNLOAD" && mkdir -p "$DIR_DOWNLOAD" && chmod 777 "$DIR_DOWNLOAD"
		__SYSTEM_PREPARING
		__AMD64_DPKG_UPDATING
	fi
	__AMD64_BASE_DOWNLOADING
	__AMD64_BASE_INSTALLING
	__AMD64_WMS_INSTALLING
	echo "deb https://wmp.wildix.com/debian/get_w.php?want= rel40dev main" > /etc/apt/sources.list

	__COLOR "\n------------------------" "bold"
	__COLOR "Upgrade from 32 to 64: done" "yellow_l"
	__COLOR "------------------------" "bold"
	__COLOR "Press any key to reboot:" "inverted" "n"
	read ans
	__COLOR "reboot..." "bold"
	sleep 5 && reboot
else
	__COLOR "Your system already have amd64 architecture (exit)" "green_l"
fi