#!/bin/bash

APP=$(basename $0)
ERL_LOG="/var/log/ejabberd/crash.log"
WORKPATH='/tmp/erldata'

log () {
  logger -p syslog.info -t $APP "$1"
}

# get package version
get_version () {
    echo -n $(dpkg-query -W -f='${Version}' "ejabberd")
}

TIMESTAMP=$(date --rfc-3339=seconds)

mkdir -p $WORKPATH

NAMENS=$(date +%N)
ARCHIVE="${WORKPATH}/erlang-${NAMENS}.zip"

if [ -s $ERL_LOG ]; then
    cat $ERL_LOG | zip -q -1 $ARCHIVE -
    echo -ne "@ -\n@=crash.log\n" | zipnote -w $ARCHIVE
else
    exit 0
fi

cd $WORKPATH

VERSION=$(get_version $PATHNAME)

JSON="{\n  \"process\": \"ejabberd\",\n  \"path\": \"ejabberd\",\n  \"signal\": 0,\n  \"timestamp\": \"$TIMESTAMP\",\n  \"created\": \"$(date --rfc-3339=seconds)\",\n  \"version\": \"$VERSION\",\n  \"build_id\": \"deadbeaf\"\n}"
echo -e "$JSON" > ${WORKPATH}/core.json
zip -gjqm -1 $ARCHIVE ${WORKPATH}/core.json

chown wms:wms $ARCHIVE
log "Ejabberd crash log archive is ready to send"

doit () {
  sleep $((RANDOM % 4200))
  /usr/sbin/send_crash_dump $ARCHIVE > /dev/null 2>&1
}

doit &

exit 0
